#include "aes.h"
#include <io.h>
/*
  @author Ralf Ramsauer, ralf@humppa.name
 */
/* 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


static unsigned char aes_perform_block( void *destination, const void *source );

void aes_reset(void)
{
  AESACTL0 |= AESSWRST; // set AES software reset bit
  while( AESACTL0 & AESSWRST ); // wait while bit is set
}

unsigned char aes_setkey( void *key )
{
  AESASTAT &= ~AESKEYWR;
  
  // Now write the 128Bit Key into the AESADIN Register
  unsigned char i;
  #if AES_ACCESS_MODE == AES8BIT
  for( i=0 ; i < 16 ; i++ )
  {
    AESAKEY_L = ((unsigned char*)key)[i];
    while( ((AESASTAT>>4)&0x000f) != ((i+1)&0xf) ); // Wait until value is loaded	
  }
  #elif AES_ACCESS_MODE == AES16BIT
  for( i=0 ; i < 8 ; i++ )
  {
    AESAKEY = ((unsigned int*)key)[i];
    while( ((AESASTAT>>4)&0x000f) != (((i+1)*2)&0xf) ); // Wait until value is loaded
  }
  #endif
  if( AESACTL0 & AESERRFG )
    return AES_ERROR;
  while( (AESASTAT & AESKEYWR) == 0 );  
  return AES_SUCCESS;
}

unsigned char aes_encrypt( void *destination, const void *source, unsigned int blocks )
{
  unsigned int tblocks;
  AESACTL0 &= ~(AESOP0|AESOP1); // Set AES Mode: Encryption

  for( tblocks = 0 ; tblocks < blocks ; tblocks++ )
  {
    aes_perform_block( destination+tblocks*16, source+tblocks*16 );
  }

  return AES_SUCCESS;
}

unsigned char aes_decrypt( void *destination, const void *source, unsigned int blocks )
{
  unsigned int tblocks;
  AESACTL0 |= AESOP0; AESACTL0 &= ~(AESOP1); // Set AES Mode: Decryption

  for( tblocks = 0 ; tblocks < blocks ; tblocks++ )
  {
    aes_perform_block( destination+tblocks*16, source+tblocks*16 );
  }

  return AES_SUCCESS;
}

static unsigned char aes_perform_block( void *destination, const void *source )
{
  unsigned char i;
  AESASTAT |= AESKEYWR; // Use present Key
  #if AES_ACCESS_MODE == AES8BIT
  for(i=0;i<16;i++) // Load the data 8bit wise
  {
    AESADIN_L = ((unsigned char*)source)[i];
    while( ((AESASTAT>>8)&0x000f) != ((i+1)&0xf) );
  }
  #elif AES_ACCESS_MODE == AES16BIT
  for(i=0;i<8;i++) // Load the data 16bit wise
  {
    AESADIN = ((unsigned int*)source)[i];
    while( ((AESASTAT>>8)&0x000f) != (((i+1)*2)&0xf) );
  }
  #endif
  AESASTAT |= AESDINWR; // Tell the controller, that the data is loaded
  // Now the Controller automatically starts encrypting/decrypting
  while( AESASTAT & AESBUSY ); // busy wait while it's encrypting
  while( (AESACTL0 & AESRDYIFG) == 0 );// Fetch the IFG
  
  // Write the encrypted data back to destination
  #if AES_ACCESS_MODE == AES8BIT
  for( i = 0 ; i < 16 ; i++ )
  {
    ((unsigned char*)destination)[i] = AESADOUT_L;
    while( ((AESASTAT>>12)&0x000f) != ((i+1)&0xf) );
  }
  #elif AES_ACCESS_MODE == AES16BIT
  for( i = 0 ; i < 8 ; i++ )
  {
    ((unsigned int*)destination)[i] = AESADOUT;
    while( ((AESASTAT>>12)&0x000f) != (((i+1)*2)&0xf) );
  }
  #endif
  if( AESACTL0 & AESERRFG )
    return AES_ERROR;
  return AES_SUCCESS;
}
